@@ -11,10 +11,11 @@
 
 // Start of defines instantiated by CMake.
 // clang-format off
-#define MPPP_VERSION 0.4
+#define MPPP_VERSION 0.5
 #define MPPP_VERSION_MAJOR 0
-#define MPPP_VERSION_MINOR 4
+#define MPPP_VERSION_MINOR 5
 #define MPPP_WITH_MPFR
+
 // clang-format on
 // End of defines instantiated by CMake.
 
@@ -48,7 +49,7 @@
 
 #endif
 
-// Configuration of the thread_local keyword.
+// thread_local configuration.
 #if defined(__apple_build_version__) || defined(__MINGW32__) || defined(__INTEL_COMPILER)
 
 // - Apple clang does not support the thread_local keyword until very recent versions.
@@ -59,8 +60,8 @@
 #else
 
 // For the rest, we assume thread_local is available.
-#define MPPP_HAVE_THREAD_LOCAL
 #define MPPP_MAYBE_TLS static thread_local
+#define MPPP_HAVE_THREAD_LOCAL
 
 #endif
 
@@ -71,4 +72,28 @@
 
 #endif
 
+// C++ standard setup.
+// NOTE: this is necessary because at this time MSVC does not set correctly the
+// __cplusplus macro.
+#if defined(_MSC_VER)
+
+#define MPPP_CPLUSPLUS _MSVC_LANG
+
+#else
+
+#define MPPP_CPLUSPLUS __cplusplus
+
+#endif
+
+// constexpr setup.
+#if MPPP_CPLUSPLUS >= 201402L
+
+#define MPPP_CONSTEXPR_14 constexpr
+
+#else
+
+#define MPPP_CONSTEXPR_14
+
+#endif
+
 #endif
