@@ -20,6 +20,13 @@ inline namespace detail
 // A bunch of useful utilities from C++14/C++17.
 
 // http://en.cppreference.com/w/cpp/types/void_t
+#if MPPP_CPLUSPLUS >= 201703L
+
+template <typename... Ts>
+using void_t = std::void_t<Ts...>;
+
+#else
+
 template <typename... Ts>
 struct make_void {
     typedef void type;
@@ -28,6 +35,8 @@ struct make_void {
 template <typename... Ts>
 using void_t = typename make_void<Ts...>::type;
 
+#endif
+
 // http://en.cppreference.com/w/cpp/experimental/is_detected
 template <class Default, class AlwaysVoid, template <class...> class Op, class... Args>
 struct detector {
@@ -56,6 +65,21 @@ template <template <class...> class Op, class... Args>
 using detected_t = typename detector<nonesuch, void, Op, Args...>::type;
 
 // http://en.cppreference.com/w/cpp/types/conjunction
+// http://en.cppreference.com/w/cpp/types/disjunction
+// http://en.cppreference.com/w/cpp/types/negation
+#if MPPP_CPLUSPLUS >= 201703L
+
+template <typename... Ts>
+using conjunction = std::conjunction<Ts...>;
+
+template <typename... Ts>
+using disjunction = std::disjunction<Ts...>;
+
+template <typename T>
+using negation = std::negation<T>;
+
+#else
+
 template <class...>
 struct conjunction : std::true_type {
 };
@@ -68,7 +92,6 @@ template <class B1, class... Bn>
 struct conjunction<B1, Bn...> : std::conditional<B1::value != false, conjunction<Bn...>, B1>::type {
 };
 
-// http://en.cppreference.com/w/cpp/types/disjunction
 template <class...>
 struct disjunction : std::false_type {
 };
@@ -81,21 +104,53 @@ template <class B1, class... Bn>
 struct disjunction<B1, Bn...> : std::conditional<B1::value != false, B1, disjunction<Bn...>>::type {
 };
 
-// http://en.cppreference.com/w/cpp/types/negation
 template <class B>
 struct negation : std::integral_constant<bool, !B::value> {
 };
 
-// Some handy aliases.
-template <typename T>
-using uncvref_t = typename std::remove_cv<typename std::remove_reference<T>::type>::type;
+#endif
 
 // Small helpers, like C++14.
+#if MPPP_CPLUSPLUS >= 201402L
+
+template <bool B, typename T = void>
+using enable_if_t = std::enable_if_t<B, T>;
+
+template <typename T>
+using make_unsigned_t = std::make_unsigned_t<T>;
+
+template <typename T>
+using remove_cv_t = std::remove_cv_t<T>;
+
+template <typename T>
+using remove_extent_t = std::remove_extent_t<T>;
+
+#else
+
 template <bool B, typename T = void>
 using enable_if_t = typename std::enable_if<B, T>::type;
 
 template <typename T>
-using make_unsigned = typename std::make_unsigned<T>::type;
+using make_unsigned_t = typename std::make_unsigned<T>::type;
+
+template <typename T>
+using remove_cv_t = typename std::remove_cv<T>::type;
+
+template <typename T>
+using remove_extent_t = typename std::remove_extent<T>::type;
+
+#endif
+
+// Some handy aliases.
+template <typename T>
+using unref_t = typename std::remove_reference<T>::type;
+
+template <typename T>
+using uncvref_t = remove_cv_t<unref_t<T>>;
+
+// Detect non-const rvalue references.
+template <typename T>
+using is_ncrvr = conjunction<std::is_rvalue_reference<T>, negation<std::is_const<unref_t<T>>>>;
 }
 }
 
